@@ -1,4 +1,7 @@
-## ------------------------------------------------------------------------
+## ---- results='hide', echo=FALSE----------------------------------------------
+set.seed(0)
+
+## -----------------------------------------------------------------------------
 n <- 200
 m1 <- matrix(runif(n*n),ncol=n)
 b1 <- runif(n)
@@ -6,7 +9,7 @@ if (requireNamespace("microbenchmark", quietly = TRUE)) {
   microbenchmark::microbenchmark(solve(m1, b1), m1 %*% b1)
 }
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 set.seed(0)
 corr <- function(x,y) {exp(sum(-30*(x-y)^2))}
 n <- 200
